#!/usr/bin/python3

# Copyright (C) <2015-present>  <it-novum GmbH>
#
# This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
# The license agreement and license key were sent with the order confirmation.

import requests
import urllib3
import sys
import argparse
import json
import os
import time
from sys import exit
from pprint import pprint
from datetime import datetime
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

parser = argparse.ArgumentParser()
optional = parser._action_groups.pop()
required = parser.add_argument_group('required arguments')
required.add_argument('--hostname', help='hostnamae or ipadresse [single Host|Loadbalancer]', required=True)
required.add_argument('--username', help='Horizon username', required=True)
required.add_argument('--password', help='Horizon password', required=True)
required.add_argument('--domain', help='Horizon domain', required=True)
required.add_argument('--mode', help='Check function [connections, overallstatus, sessions, replication, certificate, database, gateway-status, gateway-connections]', required=True)
optional.add_argument('--warning', help='warning threshold', required=False)
optional.add_argument('--critical', help='critical threshold', required=False)
optional.add_argument('--name', help='single host to check, otherwise it will check all hosts', required=False)
parser._action_groups.append(optional) # added this line
args=parser.parse_args()


hor_domain=args.domain
hor_user=args.username
hor_pw=args.password
hor_url="https://" + args.hostname
hor_name=args.name
check_mode=args.mode
warning=args.warning
critical=args.critical

tokenpath="/var/tmp/check_vmware_horizon"
tokenfile=tokenpath + "/" + hor_domain + "_" + hor_user + "-" + args.hostname + ".token"
maxage=900
RCtxt={0: 'OK', 1: 'WARNING', 2: 'CRITICAL', 3: 'UNKNOWN'}
OUTtxt=""
PERFtxt=""
RCok=0
RCwarning=1
RCcritical=2
RCunknown=3
RC=RCok



def horizon_login(hor_tstatus):
    atoken=""
    if not os.path.exists(tokenpath):
        os.makedirs(tokenpath)

    if os.path.exists(tokenfile) and ((time.time() - os.stat(tokenfile).st_mtime) < maxage) and hor_tstatus == "":
        with open(tokenfile, 'r') as f:
            try:
                token=json.loads(f.read())
                atoken=token['access_token']
                f.close()
                return atoken
            except:
                atoken=""
   
    if atoken == "":
        try:
            data={"username":hor_user,"password":hor_pw,"domain":hor_domain}
            r = requests.post(hor_url + "/rest/login", json=data, verify=False)
        except requests.exceptions.RequestException as e:
            print(e)
            exit(RCunknown)
        try:
            with open(tokenfile, 'w') as f:
                f.write(json.dumps(r.json()))
                f.close()
            return r.json()['access_token']
        except:
            print(r.json())
            exit(RCunknown)

def horizon_query(hor_func):
    hor_atoken=horizon_login("")
    headers = {'Authorization': 'Bearer ' + hor_atoken}
    r = requests.get(hor_url + hor_func, headers=headers, verify=False)
    if (r.status_code != requests.codes.ok):
        hor_atoken=horizon_login("UNAUTHORIZED")
        headers = {'Authorization': 'Bearer ' + hor_atoken}
        r = requests.get(hor_url + hor_func, headers=headers, verify=False)
        if (r.status_code != requests.codes.ok):
            print("UNKNOWN - could not query data")
            exit(RCunknown)
        else:
            return r.json()
    else:
        return r.json()


def check_thresholds(value, warning, critical):
    if warning is None or critical is None:
        print("UNKNOWN - please provide thresholds --warning and --critical")
        exit(RCunknown)
    try:
        warning=int(warning)
        critical=int(critical)
    except:
        print("UNKNOWN - thresholds are not numeric")
        exit(RCunknown)
    if warning < critical:
        if value < warning:
            return RCok
        elif value > critical:
            return RCcritical
        else:
            return RCwarning
    else:
        if value > warning:
            return RCok
        elif value > critical:
            return RCwarning
        else:
            return RCcritical



if check_mode == "connections":
    hor_data=horizon_query("/rest/monitor/connection-servers")
    for val in hor_data:
        if hor_name is not None:
           if hor_name.lower() in str(val['name']).lower():
                if check_thresholds(val['connection_count'], warning, critical) >= RC:
                    RC = check_thresholds(val['connection_count'], warning, critical)
                OUTtxt=OUTtxt + ("%s: %d, " %(val['name'], val['connection_count']))
                PERFtxt=PERFtxt + ("con=%d;%d;%d " %(val['connection_count'], int(warning), int(critical)))
        else:
            if check_thresholds(val['connection_count'], warning, critical) >= RC:
                RC = check_thresholds(val['connection_count'], warning, critical)
            OUTtxt=OUTtxt + ("%s: %d, " %(val['name'], val['connection_count']))
            PERFtxt=PERFtxt + ("con_%s=%d;%d;%d " %(val['name'], val['connection_count'], int(warning), int(critical)))


    print("%s - Connections: %s | %s" %(RCtxt[RC], OUTtxt[:-2], PERFtxt))

elif check_mode == "overallstatus":
    hor_data=horizon_query("/rest/monitor/connection-servers")
    for val in hor_data:
        if hor_name is not None:
           if hor_name.lower() in str(val['name']).lower():
                if str(val['status']).lower() != "ok":
                    RC=RCcritical
                OUTtxt=OUTtxt + ("%s: %s, " %(val['name'], val['status']))
        else:
            if str(val['status']).lower() != "ok":
                RC=RCcritical
            OUTtxt=OUTtxt + ("%s: %s, " %(val['name'], val['status']))
        


    print("%s - overall status: %s" %(RCtxt[RC], OUTtxt[:-2]))

elif check_mode == "sessions":
    hor_data=horizon_query("/rest/monitor/connection-servers")
    for val in hor_data:
        if 'session_protocol_data' in val:
            if hor_name is not None:
                if hor_name.lower() in str(val['name']).lower():
                    for sess in val['session_protocol_data']:
                        if check_thresholds(sess['session_count'], warning, critical) >= RC:
                            RC=check_thresholds(sess['session_count'], warning, critical)
                        OUTtxt=OUTtxt + ("%s_%s: %d, " %(val['name'], sess['session_protocol'], sess['session_count']))
                        PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], sess['session_protocol'], sess['session_count'], int(warning), int(critical)))
            else:
                for sess in val['session_protocol_data']:
                    if check_thresholds(sess['session_count'], warning, critical) >= RC:
                        RC=check_thresholds(sess['session_count'], warning, critical)
                    OUTtxt=OUTtxt + ("%s_%s: %d, " %(val['name'], sess['session_protocol'], sess['session_count']))
                    PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], sess['session_protocol'], sess['session_count'], int(warning), int(critical)))
    
    print("%s - session count: %s | %s" %(RCtxt[RC], OUTtxt[:-2], PERFtxt))

elif check_mode == "replication":
    hor_data=horizon_query("/rest/monitor/connection-servers")
    for val in hor_data:
        if 'cs_replications' in val:
            if hor_name is not None:
                if hor_name.lower() in str(val['name']).lower():
                    OUTtxt=OUTtxt + ("%s (" %(val['name']))
                    for repl in val['cs_replications']:
                        if repl['status'].lower() != "ok":
                            RC=RCcritical                    
                        OUTtxt=OUTtxt + ("%s:%s, " %(repl['server_name'], repl['status']))
                    OUTtxt=OUTtxt[:-2] + "), "
            else:
                OUTtxt=OUTtxt + ("%s (" %(val['name']))
                for repl in val['cs_replications']:
                    if repl['status'].lower() != "ok":
                        RC=RCcritical                    
                    OUTtxt=OUTtxt + ("%s:%s, " %(repl['server_name'], repl['status']))
                OUTtxt=OUTtxt[:-2] + "), "
    print("%s - Replication Status: %s" %(RCtxt[RC], OUTtxt[:-2]))


elif check_mode == "certificate":
    hor_data=horizon_query("/rest/monitor/connection-servers")
    for val in hor_data:
        if 'certificate' in val:
            if hor_name is not None:
                if hor_name.lower() in str(val['name']).lower():
                    curr_stamp=datetime.now()
                    cert_stamp=datetime.fromtimestamp(int(val['certificate']['valid_to']) / 1000)
                    time_diff=cert_stamp - curr_stamp
                    time_diff=time_diff.days
                    if check_thresholds(time_diff, warning, critical) >= RC:
                        RC=check_thresholds(time_diff, warning, critical)
                    OUTtxt=OUTtxt + ("%s: %s, " %(val['name'], cert_stamp))
            else:
                curr_stamp=datetime.now()
                cert_stamp=datetime.fromtimestamp(int(val['certificate']['valid_to']) / 1000)
                time_diff=cert_stamp - curr_stamp
                time_diff=time_diff.days
                if check_thresholds(time_diff, warning, critical) >= RC:
                    RC=check_thresholds(time_diff, warning, critical)
                OUTtxt=OUTtxt + ("%s: %s, " %(val['name'], cert_stamp))
    print("%s - certificate will expire at: %s" %(RCtxt[RC], OUTtxt[:-2]))


elif check_mode == "database":
    hor_data=horizon_query("/rest/monitor/event-database")
    if hor_data['status'].upper() == "CONNECTED":
        RC=RCok
    elif hor_data['status'].upper() in ("CONNECTING", "RECONNECTING"):
        RC=RCwarning
    elif hor_data['status'].upper() in ("DISCONNECTED", "ERROR"):
        RC=RCcritical
    else:
        RC=RCunknown
    
    print("%s - database status is: %s (%s@%s)" %(RCtxt[RC], hor_data['status'], hor_data['details']['user_name'], hor_data['details']['server_name']))
    
elif check_mode == "gateway-status":
    hor_data=horizon_query("/rest/monitor/gateways")
    for val in hor_data:
        if hor_name is not None:
           if hor_name.lower() in str(val['name']).lower():
                if val['status'].lower() != "ok":
                    RC=RCcritical
                OUTtxt=OUTtxt + ("%s: %s, " %(val['name'], val['status']))
        else:
            if val['status'].lower() != "ok":
                RC=RCcritical
            OUTtxt=OUTtxt + ("%s: %s, " %(val['name'], val['status']))
    print("%s - Gateway Status: %s" %(RCtxt[RC], OUTtxt[:-2]))

elif check_mode == "gateway-connections":
    hor_data=horizon_query("/rest/monitor/gateways")
    for val in hor_data:
        if hor_name is not None:
            if hor_name.lower() in str(val['name']).lower():
                if check_thresholds(val['active_connection_count'], warning, critical) >= RC or check_thresholds(val['blast_connection_count'], warning, critical) >= RC or check_thresholds(val['pcoip_connection_count'], warning, critical) >= RC:
                    RC=max(check_thresholds(val['active_connection_count'], warning, critical), check_thresholds(val['blast_connection_count'], warning, critical), check_thresholds(val['pcoip_connection_count'], warning, critical))
                OUTtxt=OUTtxt + ("%s: (active %s, blast %s, pcoip %s), " %(val['name'], val['active_connection_count'], val['blast_connection_count'], val['pcoip_connection_count']))
                PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], "active", val['active_connection_count'], int(warning), int(critical)))
                PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], "blast", val['blast_connection_count'], int(warning), int(critical)))
                PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], "pcoip", val['pcoip_connection_count'], int(warning), int(critical)))
        else:
            if check_thresholds(val['active_connection_count'], warning, critical) >= RC or check_thresholds(val['blast_connection_count'], warning, critical) >= RC or check_thresholds(val['pcoip_connection_count'], warning, critical) >= RC:
                RC=max(check_thresholds(val['active_connection_count'], warning, critical), check_thresholds(val['blast_connection_count'], warning, critical), check_thresholds(val['pcoip_connection_count'], warning, critical))
            OUTtxt=OUTtxt + ("%s: (active %s, blast %s, pcoip %s), " %(val['name'], val['active_connection_count'], val['blast_connection_count'], val['pcoip_connection_count']))
            PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], "active", val['active_connection_count'], int(warning), int(critical)))
            PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], "blast", val['blast_connection_count'], int(warning), int(critical)))
            PERFtxt=PERFtxt + ("%s_%s=%d;%d;%d " %(val['name'], "pcoip", val['pcoip_connection_count'], int(warning), int(critical)))

    print("%s - Gateway Connections: %s | %s" %(RCtxt[RC], OUTtxt[:-2], PERFtxt))

elif check_mode == "desktop-pools":
    hor_data=horizon_query("/rest/inventory/v1/desktop-pools")
    transstatus=""
    for val in hor_data:
        if hor_name is not None:
            if hor_name.lower() in str(val['name']).lower():
                if val['enabled'] is False:
                    transstatus="disabled"
                    RC=RCcritical
                elif val['enabled'] is True:
                    transstatus="enabled"
                OUTtxt=OUTtxt + ("%s: %s, "%(val['name'], transstatus))
        else:
            if val['enabled'] is False:
                transstatus="disabled"
                RC=RCcritical
            elif val['enabled'] is True:
                transstatus="enabled"
            OUTtxt=OUTtxt + ("%s: %s, "%(val['name'], transstatus))
        
    print("%s - Desktop Pools: %s" %(RCtxt[RC], OUTtxt[:-2]))
    
else:
    print("%s - mode %s not available" %(RCtxt[3], check_mode))
    RC=RCunknown

exit(RC)

